@@ -381,7 +381,7 @@ notify_dispatch_session(int fd, short event, void *arg)
 
 	if (event & EV_READ) {
 		if ((n = imsgbuf_read(ibuf)) == -1)
-			fatal("imsg_read error");
+			fatal("imsgbuf_read error");
 		if (n == 0) {
 			/* Connection closed. */
 			shut = 1;
@@ -515,7 +515,7 @@ notify_dispatch(int fd, short event, void *arg)
 
 	if (event & EV_READ) {
 		if ((n = imsgbuf_read(imsgbuf)) == -1)
-			fatal("imsg_read error");
+			fatal("imsgbuf_read error");
 		if (n == 0) {
 			/* Connection closed. */
 			shut = 1;
