@@ -142,13 +142,13 @@ gotwebd_dispatch_sockets(int fd, short event, void *arg)
 
 	if (event & EV_READ) {
 		if ((n = imsgbuf_read(ibuf)) == -1)
-			fatal("imsg_read error");
+			fatal("imsgbuf_read error");
 		if (n == 0)	/* Connection closed */
 			shut = 1;
 	}
 	if (event & EV_WRITE) {
 		if (imsgbuf_write(ibuf) == -1)
-			fatal("msgbuf_write");
+			fatal("imsgbuf_write");
 	}
 
 	for (;;) {
@@ -270,6 +270,7 @@ int
 main(int argc, char **argv)
 {
 	struct event		 sigint, sigterm, sighup, sigpipe, sigusr1;
+	struct event_base	*evb;
 	struct gotwebd		*env;
 	struct passwd		*pw;
 	int			 ch, i;
@@ -367,7 +368,7 @@ main(int argc, char **argv)
 	if (!env->gotwebd_debug && daemon(1, 0) == -1)
 		fatal("daemon");
 
-	event_init();
+	evb = event_init();
 
 	env->nserver = env->prefork_gotwebd;
 	env->iev_server = calloc(env->nserver, sizeof(*env->iev_server));
@@ -421,6 +422,7 @@ main(int argc, char **argv)
 #endif
 
 	event_dispatch();
+	event_base_free(evb);
 
 	log_debug("%s gotwebd exiting", getprogname());
 
@@ -485,6 +487,7 @@ gotwebd_shutdown(void)
 		close(env->iev_server[i].ibuf.fd);
 		env->iev_server[i].ibuf.fd = -1;
 	}
+	free(env->iev_server);
 
 	do {
 		pid = waitpid(WAIT_ANY, &status, 0);
@@ -499,6 +502,21 @@ gotwebd_shutdown(void)
 			    pid);
 	} while (pid != -1 || (pid == -1 && errno == EINTR));
 
+	while (!TAILQ_EMPTY(&gotwebd_env->addresses)) {
+		struct address *h;
+
+		h = TAILQ_FIRST(&gotwebd_env->addresses);
+		TAILQ_REMOVE(&gotwebd_env->addresses, h, entry);
+		free(h);
+	}
+	while (!TAILQ_EMPTY(&gotwebd_env->servers)) {
+		struct server *srv;
+
+		srv = TAILQ_FIRST(&gotwebd_env->servers);
+		TAILQ_REMOVE(&gotwebd_env->servers, srv, entry);
+		free(srv);
+	}
+	sockets_purge(gotwebd_env);
 	free(gotwebd_env);
 
 	log_warnx("gotwebd terminating");
