@@ -263,13 +263,14 @@ escape_path(char *buf, size_t bufsize, const char *path)
 
 const struct got_error *
 got_dial_ssh(pid_t *newpid, int *newfd, const char *host,
-    const char *port, const char *path, const char *command, int verbosity)
+    const char *port, const char *path, const char *jumphost,
+    const char *command, int verbosity)
 {
 	const struct got_error *error = NULL;
 	int pid, pfd[2];
 	char cmd[64];
 	char escaped_path[PATH_MAX];
-	const char *argv[11];
+	const char *argv[13];
 	int i = 0, j;
 
 	*newpid = -1;
@@ -291,6 +292,10 @@ got_dial_ssh(pid_t *newpid, int *newfd, const char *host,
 		for (j = 0; j < MIN(3, verbosity); j++)
 			argv[i++] = "-v";
 	}
+	if (jumphost) {
+		argv[i++] = "-J";
+		argv[i++] = jumphost;
+	}
 	argv[i++] = "--";
 	argv[i++] = (char *)host;
 	argv[i++] = (char *)cmd;
