@@ -121,7 +121,7 @@ get_diffstat(struct got_diffstat_cb_arg *ds, const char *path,
 			return err;
 	}
 
-	pe = TAILQ_LAST(ds->paths, got_pathlist_head);
+	pe = RB_MAX(got_pathlist_head, ds->paths);
 	diffstat_field_width(&ds->max_path_len, &ds->add_cols, &ds->rm_cols,
 	    pe->path_len, change->add, change->rm);
 
@@ -219,8 +219,10 @@ diff_blobs(struct got_diff_line **lines, size_t *nlines,
 		}
 		n = fprintf(outfile, "blob - %s%s\n", idstr1,
 		    modestr1 ? modestr1 : "");
-		if (n < 0)
+		if (n < 0) {
+			err = got_error_from_errno("fprintf");
 			goto done;
+		}
 		outoff += n;
 		if (lines && *lines) {
 			err = add_line_metadata(lines, nlines, outoff,
@@ -231,8 +233,10 @@ diff_blobs(struct got_diff_line **lines, size_t *nlines,
 
 		n = fprintf(outfile, "blob + %s%s\n", idstr2,
 		    modestr2 ? modestr2 : "");
-		if (n < 0)
+		if (n < 0) {
+			err = got_error_from_errno("fprintf");
 			goto done;
+		}
 		outoff += n;
 		if (lines && *lines) {
 			err = add_line_metadata(lines, nlines, outoff,
@@ -243,7 +247,7 @@ diff_blobs(struct got_diff_line **lines, size_t *nlines,
 	}
 
 	err = got_diffreg(&result, f1, f2, diff_algo, ignore_whitespace,
-	     force_text_diff);
+	    force_text_diff);
 	if (err)
 		goto done;
 
@@ -342,20 +346,33 @@ got_diff_blob(struct got_diff_line **lines, size_t*nlines,
 }
 
 static const struct got_error *
-diff_blob_file(struct got_diffreg_result **resultp,
-    struct got_blob_object *blob1, FILE *f1, off_t size1, const char *label1,
-    FILE *f2, int f2_exists, struct stat *sb2, const char *label2,
-    enum got_diff_algorithm diff_algo, int diff_context, int ignore_whitespace,
-    int force_text_diff, struct got_diffstat_cb_arg *diffstat, FILE *outfile)
+diff_blob_file(struct got_diff_line **lines, size_t *nlines,
+    struct got_diffreg_result **resultp, struct got_blob_object *blob1,
+    FILE *f1, off_t size1, const char *label1, FILE *f2, int f2_exists,
+    struct stat *sb2, const char *label2, enum got_diff_algorithm diff_algo,
+    int diff_context, int ignore_whitespace, int force_text_diff,
+    struct got_diffstat_cb_arg *diffstat, FILE *outfile)
 {
 	const struct got_error *err = NULL, *free_err;
 	char hex1[GOT_OBJECT_ID_HEX_MAXLEN];
 	const char *idstr1 = NULL;
 	struct got_diffreg_result *result = NULL;
+	off_t outoff = 0;
+	int n;
 
 	if (resultp)
 		*resultp = NULL;
 
+	if (lines != NULL && *lines != NULL) {
+		if (*nlines == 0) {
+			err = add_line_metadata(lines, nlines, 0,
+			    GOT_DIFF_LINE_NONE);
+			if (err != NULL)
+				return err;
+		} else
+			outoff = (*lines)[*nlines - 1].offset;
+	}
+
 	if (blob1)
 		idstr1 = got_object_blob_id_str(blob1, hex1, sizeof(hex1));
 	else
@@ -364,6 +381,17 @@ diff_blob_file(struct got_diffreg_result **resultp,
 	if (outfile) {
 		char	*mode = NULL;
 
+		n = fprintf(outfile, "blob - %s\n", label1 ? label1 : idstr1);
+		if (n < 0)
+			return got_error_from_errno("fprintf");
+		if (lines != NULL && *lines != NULL) {
+			outoff += n;
+			err = add_line_metadata(lines, nlines, outoff,
+			    GOT_DIFF_LINE_BLOB_MIN);
+			if (err != NULL)
+				return err;
+		}
+
 		/* display file mode for new added files only */
 		if (f2_exists && blob1 == NULL) {
 			int mmask = (S_IRWXU | S_IRWXG | S_IRWXO);
@@ -374,26 +402,35 @@ diff_blob_file(struct got_diffreg_result **resultp,
 			    sb2->st_mode & mmask) == -1)
 				return got_error_from_errno("asprintf");
 		}
-		fprintf(outfile, "blob - %s\n", label1 ? label1 : idstr1);
-		fprintf(outfile, "file + %s%s\n",
+		n = fprintf(outfile, "file + %s%s\n",
 		    f2_exists ? label2 : "/dev/null", mode ? mode : "");
 		free(mode);
+		if (n < 0)
+			return got_error_from_errno("fprintf");
+		if (lines != NULL && *lines != NULL) {
+			outoff += n;
+			err = add_line_metadata(lines, nlines, outoff,
+			    GOT_DIFF_LINE_BLOB_PLUS);
+			if (err != NULL)
+				return err;
+		}
 	}
 
 	err = got_diffreg(&result, f1, f2, diff_algo, ignore_whitespace,
 	    force_text_diff);
 	if (err) {
 		char msg[GOT_ERR_MAX_MSG_SIZE];
+
 		if (snprintf(msg, sizeof(msg), "%s vs %s: %s",
-		             label1 ? label1 : idstr1,
-		             f2_exists ? label2 : "/dev/null", err->msg) >= 0) {
+		    label1 ? label1 : idstr1,
+		    f2_exists ? label2 : "/dev/null", err->msg) >= 0) {
 			err = got_error_msg(err->code, msg);
 		}
 		goto done;
 	}
 
 	if (outfile) {
-		err = got_diffreg_output(NULL, NULL, result,
+		err = got_diffreg_output(lines, nlines, result,
 		    blob1 != NULL, f2_exists,
 		    label2, /* show local file's path, not a blob ID */
 		    label2, GOT_DIFF_OUTPUT_UNIDIFF,
@@ -448,15 +485,15 @@ done:
 }
 
 const struct got_error *
-got_diff_blob_file(struct got_blob_object *blob1, FILE *f1, off_t size1,
-    const char *label1, FILE *f2, int f2_exists, struct stat *sb2,
-    const char *label2, enum got_diff_algorithm diff_algo, int diff_context,
-    int ignore_whitespace, int force_text_diff,
-    struct got_diffstat_cb_arg *ds, FILE *outfile)
+got_diff_blob_file(struct got_diff_line **lines, size_t *nlines,
+    struct got_blob_object *blob1, FILE *f1, off_t size1, const char *label1,
+    FILE *f2, int f2_exists, struct stat *sb2, const char *label2,
+    enum got_diff_algorithm diff_algo, int diff_context, int ignore_whitespace,
+    int force_text_diff, struct got_diffstat_cb_arg *ds, FILE *outfile)
 {
-	return diff_blob_file(NULL, blob1, f1, size1, label1, f2, f2_exists,
-	    sb2, label2, diff_algo, diff_context, ignore_whitespace,
-	    force_text_diff, ds, outfile);
+	return diff_blob_file(lines, nlines, NULL, blob1, f1, size1, label1,
+	    f2, f2_exists, sb2, label2, diff_algo, diff_context,
+	    ignore_whitespace, force_text_diff, ds, outfile);
 }
 
 static const struct got_error *
@@ -1086,7 +1123,7 @@ diff_paths(struct got_tree_object *tree1, struct got_tree_object *tree2,
 	struct got_tree_object *subtree1 = NULL, *subtree2 = NULL;
 	struct got_blob_object *blob1 = NULL, *blob2 = NULL;
 
-	TAILQ_FOREACH(pe, paths, entry) {
+	RB_FOREACH(pe, got_pathlist_head, paths) {
 		int type1 = GOT_OBJ_TYPE_ANY, type2 = GOT_OBJ_TYPE_ANY;
 		mode_t mode1 = 0, mode2 = 0;
 
@@ -1278,7 +1315,7 @@ diff_objects_as_trees(struct got_diff_line **lines, size_t *nlines,
 		arg.lines = NULL;
 		arg.nlines = 0;
 	}
-	if (paths == NULL || TAILQ_EMPTY(paths))
+	if (paths == NULL || RB_EMPTY(paths))
 		err = got_diff_tree(tree1, tree2, f1, f2, fd1, fd2, label1,
 		    label2, repo, got_diff_blob_output_unidiff, &arg, 1);
 	else
