@@ -74,7 +74,8 @@
 
 const struct got_error *
 got_fetch_connect(pid_t *fetchpid, int *fetchfd, const char *proto,
-    const char *host, const char *port, const char *server_path, int verbosity)
+    const char *host, const char *port, const char *server_path,
+    const char *jumphost, int verbosity)
 {
 	const struct got_error *err = NULL;
 
@@ -83,7 +84,7 @@ got_fetch_connect(pid_t *fetchpid, int *fetchfd, const char *proto,
 
 	if (strcmp(proto, "ssh") == 0 || strcmp(proto, "git+ssh") == 0)
 		err = got_dial_ssh(fetchpid, fetchfd, host, port,
-		    server_path, GOT_DIAL_CMD_FETCH, verbosity);
+		    server_path, jumphost, GOT_DIAL_CMD_FETCH, verbosity);
 	else if (strcmp(proto, "git") == 0)
 		err = got_dial_git(fetchfd, host, port, server_path,
 		    GOT_DIAL_CMD_FETCH);
@@ -130,6 +131,8 @@ got_fetch_pack(struct got_object_id **pack_hash, struct got_pathlist_head *refs,
 	char *progress = NULL;
 
 	*pack_hash = NULL;
+	memset(&fetchibuf, 0, sizeof(fetchibuf));
+	memset(&idxibuf, 0, sizeof(idxibuf));
 
 	if (repo && got_repo_get_object_format(repo) != GOT_HASH_SHA1)
 		return got_error_fmt(GOT_ERR_NOT_IMPL,
@@ -139,7 +142,7 @@ got_fetch_pack(struct got_object_id **pack_hash, struct got_pathlist_head *refs,
 	 * Prevent fetching of references that won't make any
 	 * sense outside of the remote repository's context.
 	 */
-	TAILQ_FOREACH(pe, wanted_refs, entry) {
+	RB_FOREACH(pe, got_pathlist_head, wanted_refs) {
 		const char *refname = pe->path;
 		if (strncmp(refname, "refs/got/", 9) == 0 ||
 		    strncmp(refname, "got/", 4) == 0 ||
@@ -154,7 +157,7 @@ got_fetch_pack(struct got_object_id **pack_hash, struct got_pathlist_head *refs,
 	for (i = 0; i < nitems(tmpfds); i++)
 		tmpfds[i] = -1;
 
-	TAILQ_INIT(&have_refs);
+	RB_INIT(&have_refs);
 	TAILQ_INIT(&my_refs);
 
 	if (!list_refs_only) {
@@ -166,7 +169,7 @@ got_fetch_pack(struct got_object_id **pack_hash, struct got_pathlist_head *refs,
 
 	TAILQ_FOREACH(re, &my_refs, entry) {
 		struct got_object_id *id;
-		const char *refname;
+		char *refname;
 
 		if (got_ref_is_symbolic(re->ref))
 			continue;
@@ -183,7 +186,7 @@ got_fetch_pack(struct got_object_id **pack_hash, struct got_pathlist_head *refs,
 		if (err)
 			goto done;
 		if (new == NULL){
-			free(&refname);
+			free(refname);
 			free(id);
 		}
 
@@ -542,6 +545,10 @@ got_fetch_pack(struct got_object_id **pack_hash, struct got_pathlist_head *refs,
 	tmpidxpath = NULL;
 
 done:
+	if (fetchibuf.w)
+		imsgbuf_clear(&fetchibuf);
+	if (idxibuf.w)
+		imsgbuf_clear(&idxibuf);
 	if (tmppackpath && unlink(tmppackpath) == -1 && err == NULL)
 		err = got_error_from_errno2("unlink", tmppackpath);
 	if (tmpidxpath && unlink(tmpidxpath) == -1 && err == NULL)
