@@ -82,7 +82,8 @@
 
 const struct got_error *
 got_send_connect(pid_t *sendpid, int *sendfd, const char *proto,
-    const char *host, const char *port, const char *server_path, int verbosity)
+    const char *host, const char *port, const char *server_path,
+    const char *jumphost, int verbosity)
 {
 	const struct got_error *err = NULL;
 
@@ -91,7 +92,7 @@ got_send_connect(pid_t *sendpid, int *sendfd, const char *proto,
 
 	if (strcmp(proto, "ssh") == 0 || strcmp(proto, "git+ssh") == 0)
 		err = got_dial_ssh(sendpid, sendfd, host, port, server_path,
-		    GOT_DIAL_CMD_SEND, verbosity);
+		    jumphost, GOT_DIAL_CMD_SEND, verbosity);
 	else if (strcmp(proto, "git") == 0)
 		err = got_dial_git(sendfd, host, port, server_path,
 		    GOT_DIAL_CMD_SEND);
@@ -261,16 +262,12 @@ realloc_ids(struct got_object_id ***ids, size_t *nalloc, size_t n)
 static struct got_pathlist_entry *
 find_ref(struct got_pathlist_head *refs, const char *refname)
 {
-	struct got_pathlist_entry *pe;
+	struct got_pathlist_entry find;
 
-	TAILQ_FOREACH(pe, refs, entry) {
-		if (got_path_cmp(pe->path, refname, strlen(pe->path),
-		    strlen(refname)) == 0) {
-			return pe;
-		}
-	}
+	find.path = refname;
+	find.path_len = strlen(refname);
+	return RB_FIND(got_pathlist_head, refs, &find);
 
-	return NULL;
 }
 
 static const struct got_error *
@@ -361,14 +358,16 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 	FILE *delta_cache = NULL;
 	char *s = NULL;
 
-	TAILQ_INIT(&have_refs);
-	TAILQ_INIT(&their_refs);
+	RB_INIT(&have_refs);
+	RB_INIT(&their_refs);
+
+	memset(&sendibuf, 0, sizeof(sendibuf));
 
 	if (got_repo_get_object_format(repo) != GOT_HASH_SHA1)
 		return got_error_fmt(GOT_ERR_NOT_IMPL,
 		    "sha256 object IDs unsupported in network protocol");
 
-	TAILQ_FOREACH(pe, branch_names, entry) {
+	RB_FOREACH(pe, got_pathlist_head, branch_names) {
 		const char *branchname = pe->path;
 		const char *targetname = pe->data;
 
@@ -396,7 +395,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 		s = NULL;
 	}
 
-	TAILQ_FOREACH(pe, delete_branches, entry) {
+	RB_FOREACH(pe, got_pathlist_head, delete_branches) {
 		const char *branchname = pe->path;
 		struct got_pathlist_entry *ref;
 		if (strncmp(branchname, "refs/heads/", 11) != 0) {
@@ -413,7 +412,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 		}
 	}
 
-	TAILQ_FOREACH(pe, tag_names, entry) {
+	RB_FOREACH(pe, got_pathlist_head, tag_names) {
 		const char *tagname = pe->path;
 		if (strncmp(tagname, "refs/tags/", 10) != 0) {
 			if (asprintf(&s, "refs/tags/%s", tagname) == -1) {
@@ -436,7 +435,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 		s = NULL;
 	}
 
-	if (TAILQ_EMPTY(&have_refs) && TAILQ_EMPTY(delete_branches)) {
+	if (RB_EMPTY(&have_refs) && RB_EMPTY(delete_branches)) {
 		err = got_error(GOT_ERR_SEND_EMPTY);
 		goto done;
 	}
@@ -487,7 +486,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 	 * Prepare the array of our object IDs which
 	 * will be needed for generating a pack file.
 	 */
-	TAILQ_FOREACH(pe, &have_refs, entry) {
+	RB_FOREACH(pe, got_pathlist_head,  &have_refs) {
 		struct got_object_id *id = pe->data;
 
 		err = realloc_ids(&our_ids, &nalloc_ours, nours + 1);
@@ -512,7 +511,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 	 * This array will be used to exclude objects which already
 	 * exist on the server from our pack file.
 	 */
-	TAILQ_FOREACH(pe, &their_refs, entry) {
+	RB_FOREACH(pe, got_pathlist_head, &their_refs) {
 		const char *refname = pe->path;
 		struct got_object_id *their_id = pe->data;
 		int have_their_id;
@@ -606,14 +605,14 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 	}
 
 	/* Account for any new references we are going to upload. */
-	TAILQ_FOREACH(pe, &have_refs, entry) {
+	RB_FOREACH(pe, got_pathlist_head, &have_refs) {
 		const char *refname = pe->path;
 		if (find_ref(&their_refs, refname) == NULL)
 			refs_to_send++;
 	}
 
 	/* Account for any existing references we are going to delete. */
-	TAILQ_FOREACH(pe, delete_branches, entry) {
+	RB_FOREACH(pe, got_pathlist_head, delete_branches) {
 		const char *branchname = pe->path;
 		if (find_ref(&their_refs, branchname))
 			refs_to_delete++;
@@ -699,6 +698,8 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 		free(errmsg);
 	}
 done:
+	if (sendibuf.w)
+		imsgbuf_clear(&sendibuf);
 	if (sendpid != -1) {
 		if (err)
 			got_privsep_send_stop(imsg_sendfds[0]);
