@@ -32,7 +32,11 @@ export GOT_TEST_ALGO="${GOT_TEST_ALGO:-sha1}"
 
 export LC_ALL=C
 
-export MALLOC_OPTIONS=S
+if [ -n "$MEM_CHECK" ] && [ "$MEM_CHECK" -gt 0 ] && [ "$MEM_CHECK" -lt 5 ]; then
+	export MALLOC_OPTIONS="$MEM_CHECK"S
+else
+	export MALLOC_OPTIONS=S
+fi
 
 if [ "$(date -u -r 86400 +%F 2>/dev/null)" != 1970-01-02 ]; then
 	DATECMD=
@@ -296,7 +300,7 @@ run_test()
 
 	if [ -n "$regress_run_only" ]; then
 		case "$regress_run_only" in
-		*$testfunc*) ;;
+		*$testfunc) ;;
 		*) return ;;
 		esac
 	fi
@@ -328,3 +332,34 @@ test_done()
 		echo "test failed; leaving test data in $testroot"
 	fi
 }
+
+test_memleak_done()
+{
+	local testroot="$1"
+	local result="$2"
+
+	kdump -u malloc -f $testroot/ktrace.out > $testroot/leak-report
+	if egrep -q "( got 0x|/bin/got-)" $testroot/leak-report; then
+		cat $testroot/leak-report
+		result=1
+	fi
+
+	test_done "$testroot" "$result"
+}
+
+check_memleak()
+{
+	local testroot="$1"
+
+	if [ -z "$1" ]; then
+		echo "Testroot not passed"
+		exit 1
+	fi
+
+	if [ ! -d "$testroot" ]; then
+		echo "Directory does not exist"	
+		exit 1
+	fi
+
+	echo "ktrace -d -tu -i -f $testroot/ktrace.out"
+}
